@@ -3,41 +3,102 @@ import yfinance as yf
 import numpy as np
 import plotly.subplots as sp
 import plotly.express as px
-import plotly.graph_objects as go
 from datetime import datetime, timedelta
+import sys
+
 pd.options.mode.chained_assignment = None
 
+########################
+#Progress Bar functions#
+########################
+
+def start_progress(title):
+    global progress_x
+    sys.stdout.write(title + ": [")# + "-" * 40 + "]" + chr(8) * 41)
+    sys.stdout.flush()
+    progress_x = 0
+
+def progress(x):
+    global progress_x
+    x = int(x * 40 // 100)
+    sys.stdout.write("#" * (x - progress_x))
+    sys.stdout.flush()
+    progress_x = x
+
+def end_progress():
+    sys.stdout.write("#" * (40 - progress_x) + "] DONE! \n\n")
+    sys.stdout.flush()
+
+######################
+######## Code ########
+######################
 
 class backtest:
     """ :Parameters:
 
             asset: str or list or series
                 Instruments taken into the consideration for the backtest.
+
             o_day: list of str or timestamps or series
                 Day/Days of the position opening.
+
             c_day: list of str or timestamps or series
                 Day/Days of the position closing.
+
             weights_factor: list of int or float or array-like or series default None
                 Optional list of factors which will be considered to define the weights for taken companies. By default
                 all weights are distributed equally, however if the list of factors provided the backtest will maximize
                 the weights towards the one with max weight factor. Negative weight factor will be considered as short selling.
+
             take_profit: list of float or int or series default None
                 List of values determining the level till a particular stock shall be traded.
+
             stop_loss: list of float or int or series default None
                 List of values determining the level till a particular stock shall be traded.
+
             benchmark: str default None
                 A benchmark ticker for comparison with portfolio performance
+
+            price_period_relation: str default 'O-C'
+                Instruct what part of the trading day a position shall be opened,
+                and what part of trading day it shall be closed.
+                Possible relations:
+                O-C / Open to Close prices
+                C-O / Close to Open prices
+                C-C / Close to Close prices
+                O-O / Open to Open prices
+                "Open" - the price at which a security first trades upon the opening of an exchange on a trading day.
+                "Close" - value of the last transacted price in a security before the market officially closes.
             """
 
-    def __init__(self, asset, o_day, c_day, weights_factor=None, take_profit=None, stop_loss=None, benchmark=None):
+    def __init__(self, asset, o_day, c_day, weights_factor=None, take_profit=None, stop_loss=None, benchmark=None,
+                 price_period_relation=None):
+
         self.asset = asset
         self.b_day = o_day
         self.s_day = c_day
         self.w_factor = weights_factor if weights_factor is not None else np.ones(len(asset))
-        self.tp = take_profit if take_profit is not None else 100 * np.ones(len(asset))
-        self.sl = stop_loss if stop_loss is not None else np.zeros(len(asset))
+        self.tp = take_profit if take_profit is not None else len(self.asset) * [np.inf]
+        self.sl = stop_loss if stop_loss is not None else len(self.asset) * [-np.inf]
         self.bench = benchmark
 
+        if price_period_relation is None:
+            self.p_p_p = "Open"
+            self.p_p_n = "Adj Close"
+        else:
+            if price_period_relation == 'O-C':
+                self.p_p_p = "Open"
+                self.p_p_n = "Adj Close"
+            elif price_period_relation == 'C-O':
+                self.p_p_p = "Adj Close"
+                self.p_p_n = "Open"
+            elif price_period_relation == 'C-C':
+                self.p_p_p = "Adj Close"
+                self.p_p_n = "Adj Close"
+            else:
+                self.p_p_p = "Open"
+                self.p_p_n = "Open"
+
     def benchmark_construction(self):
         backtest.security_list(self)
         min_date = min(self.security_list['start day'])
@@ -45,9 +106,9 @@ class backtest:
         df = yf.download(self.bench, start=backtest.date_plus_one(self, min_date),
                          end=backtest.date_plus_one(self, max_date), progress=False)
         df['Ticker'] = self.bench
-        df_close = df[["Ticker", "Adj Close"]]
+        df_close = df[["Ticker", self.p_p_n]]
         df_close.columns = ['ticker', 'close_price']
-        df_open = df[["Ticker", "Open"]]
+        df_open = df[["Ticker", self.p_p_p]]
         df_open.columns = ['ticker', 'open']
         open_price = df_open.groupby('ticker').first()
         em1 = pd.DataFrame()
@@ -99,17 +160,17 @@ class backtest:
         if isinstance(self.sl, pd.core.series.Series):
             self.sl = self.sl.values
 
-
         df = pd.DataFrame(
             {"company": self.asset, "start day": self.b_day, "end day": self.s_day, "weights factor": self.w_factor,
              "take profit": self.tp, "stop loss": self.sl})
         df = df.set_index(df['company'])
         for x in range(len(df.index)):
-            if df.iloc[x, 3] < 0:
+            if df.iloc[x, 3] < 0 and df.iloc[x, 4]!= np.inf and df.iloc[x,5]!= -np.inf:
                 a = df.iloc[x, 5]
                 b = df.iloc[x, 4]
                 df.iloc[x, 4] = (1 - a) + 1
                 df.iloc[x, 5] = 1 - (b - 1)
+
         self.security_list = df
         return self.security_list
 
@@ -126,12 +187,18 @@ class backtest:
             return back_to_str
 
     def consolidated_table_detailed(self):
+
         backtest.security_list(self)
+
+        start_progress("Consolidating detailed table: ")
+
         df_1 = self.security_list
         initial_df = pd.DataFrame()
         fq = pd.DataFrame(index=self.asset, data=self.b_day)
+        # check if companies are dublicatted in a list
         v = fq.groupby(fq.index) \
             .cumcount()
+        # doubled companies will have suffix
         list_new = []
         for x in range(len(v.values)):
             if v.iloc[x] != 0:
@@ -143,7 +210,11 @@ class backtest:
         max_date = max(self.security_list['end day'])
         ydata = yf.download(self.asset, start=backtest.date_plus_one(self, min_date),
                             end=backtest.date_plus_one(self, max_date), progress=False)
+        if ydata.empty:
+            raise ValueError('No trading days were provided, perhaps days off or holidays have been given')
+        #check if column names are Multiindex
         if not isinstance(ydata.columns, pd.MultiIndex):
+        #select desired start / end dates for selected companies
             for b_d, s_d, new_com in zip(df_1["start day"], df_1["end day"], list_new):
                 data = ydata.loc[b_d:s_d]
                 data['Ticker'] = new_com
@@ -155,13 +226,16 @@ class backtest:
                 data = q1.loc[b_d:s_d]
                 data['Ticker'] = new_com
                 initial_df = pd.concat([initial_df, data])
-        df_close = initial_df[["Ticker", "Adj Close"]]
+        df_close = initial_df[["Ticker", self.p_p_n]]
         df_close.columns = ['ticker', 'close_price']
-        df_open = initial_df[["Ticker", "Open"]]
+        df_open = initial_df[["Ticker", self.p_p_p]]
         df_open.columns = ['ticker', 'open']
         open_price = df_open.groupby('ticker').first()
         em1 = pd.DataFrame()
         em2 = pd.DataFrame()
+
+        progress(50)
+
         for x in open_price.index:
             get_open = open_price.loc[x]
             get_end = df_close[df_close['ticker'] == x]
@@ -179,6 +253,22 @@ class backtest:
         dc = em2.pivot_table(index=em2.index, columns='ticker', values='daily_change')
         self.asset = list_new
         backtest.security_list(self)
+        if dc.columns.tolist() != self.asset:
+            l1 = dc.columns.tolist()
+            l2 = self.asset
+            res = [x for x in l1 + l2 if x not in l1 or x not in l2]
+            self.asset = l1
+            self.security_list = self.security_list.drop(res)
+            self.b_day = self.security_list['start day'].values
+            self.s_day = self.security_list['end day'].values
+            self.w_factor = self.security_list['weights factor'].values
+            self.sl = self.security_list['stop loss'].values
+            self.tp = self.security_list['take profit'].values
+            if res ==[]:
+                pass
+            else:
+                print(f"\nNo price data found for {res} therefore was deleted from provided inputs.\n")
+
         dc = dc[self.asset]
         dc = dc.replace([np.inf, -np.inf], np.nan)
         dc = dc.fillna(0)
@@ -191,6 +281,9 @@ class backtest:
         self.auxiliar_df = aux
         self.detailed_return = dc
 
+        progress(50)
+        end_progress()
+
         return self.detailed_return
 
     def portfolio_construction(self):
@@ -198,13 +291,20 @@ class backtest:
         :return:
             Full constructed portfolio, including position length, weights factor, stop loss & take profit.
         """
+
         backtest.consolidated_table_detailed(self)
+
+        start_progress("Constructing your portfolio: ")
+
         binary_weights = self.auxiliar_df / self.auxiliar_df
         binary_weights.fillna(value=0, inplace=True)
         fac_summing = np.sum(abs(np.array(self.w_factor)))
         dist = np.array(self.w_factor) / fac_summing
         dist_df = pd.DataFrame(index=self.asset, data=self.w_factor).T
         weights_df = binary_weights * dist
+
+        progress(25)
+
         for z in weights_df.index:
             if abs(weights_df.loc[z]).sum() != 1:
                 act = weights_df.loc[z][weights_df.loc[z] != 0]
@@ -225,6 +325,9 @@ class backtest:
         new_binary_weights = aux_table_2 / aux_table_2
         new_binary_weights.fillna(value=0, inplace=True)
         weights_df = new_binary_weights * dist
+
+        progress(25)
+
         for z in weights_df.index:
             if abs(weights_df.loc[z]).sum() != 1:
                 act = weights_df.loc[z][weights_df.loc[z] != 0]
@@ -238,6 +341,8 @@ class backtest:
         port_performance['Sum'] = port_performance['Sum'] + 1
         port_performance['Accumulation'] = (port_performance['Sum'].cumprod() - 1) * 100
 
+        progress(25)
+
         q1 = port_performance.index[0] - timedelta(days=1)  # starting from 0%
         port_performance.loc[q1] = [0] * len(port_performance.columns)
         port_performance = port_performance.sort_index()
@@ -247,10 +352,13 @@ class backtest:
 
         # small addition to be able to show a table
         execution_table = self.final_portfolio.astype(float)
-        execution_table.iloc[:,:-2] = (execution_table.iloc[:,:-2] * 100)
+        execution_table.iloc[:, :-2] = (execution_table.iloc[:, :-2] * 100)
         execution_table.drop('Sum', axis=1, inplace=True)
         self.execution_table = execution_table.round(2)
 
+        progress(25)
+        end_progress()
+
     def execution(self):
         """
         :return:
@@ -263,6 +371,10 @@ class backtest:
         except:
             backtest.portfolio_construction(self)
             df_execution = self.final_portfolio
+
+        start_progress("Working on results: ")
+        progress(50)
+
         df_execution = df_execution.round(decimals=3)
         obj = self.final_portfolio
         pdr = obj['Sum'] - 1
@@ -292,14 +404,16 @@ class backtest:
         frame = pd.DataFrame({'Indicators': list_1, 'Values': list_2})
         self.stat_frame = frame
         frame = frame.to_string(index=False)
-        print(frame)
+
 
         # ----------------------------------------------------------------------- #
         # Execution plot - 2 ways
 
+
         if self.bench is not None:
             df_bench = backtest.benchmark_construction(self)
-            df_execution = pd.merge(df_execution, df_bench["Bench_Accumulation"], how='left', left_index=True, right_index=True)
+            df_execution = pd.merge(df_execution, df_bench["Bench_Accumulation"], how='left', left_index=True,
+                                    right_index=True)
             df_execution["Bench_Accumulation"] = df_execution["Bench_Accumulation"].fillna(method='ffill')
 
             # Execution plot - Accumulation with benchmark
@@ -312,7 +426,7 @@ class backtest:
                            x=df_execution_fig1.index,
                            y=df_execution_fig1["Accumulation"],
                            title="Accumulated return",
-                           hover_data=df_execution_fig1.columns[:-3])  # show all columns values excluding last 2
+                           hover_data=df_execution_fig1.columns[:-3])  # show all columns values excluding last 3
 
             fig1.update_layout(xaxis_title="Date")
             fig1.update_traces(name='Portfolio',
@@ -324,6 +438,12 @@ class backtest:
                              name="Benchmark")
 
             fig1.update_yaxes(tickprefix="%")
+
+            progress(50)
+            end_progress()
+
+            print(frame)
+
             fig1.show()
 
         else:
@@ -339,10 +459,14 @@ class backtest:
                            hover_data=df_execution_fig1.columns[:-2])  # show all columns values excluding last 2
             fig1.update_layout(xaxis_title="Date")
             fig1.update_yaxes(tickprefix="%")
+
+            progress(50)
+            end_progress()
+
+            print(frame)
+
             fig1.show()
 
-
-
     def plotting(self):
         """
         :return:
@@ -355,6 +479,9 @@ class backtest:
             backtest.portfolio_construction(self)
             df_accum = self.final_portfolio.copy()
 
+        start_progress("Working on plots: ")
+
+        progress(20)
 
         # ----------------------------------------------------------------------- #
         # Stats
@@ -377,23 +504,15 @@ class backtest:
         self.stocks_mean = obj_only_stocks.mean()
         self.top_per = self.stocks_mean.nlargest(1)
         self.worst_per = self.stocks_mean.nsmallest(1)
-        self.trade_length = len(pdr)-1
+        self.trade_length = len(pdr) - 1
         VaR_95 = -1.65 * self.port_std * np.sqrt(self.trade_length)
         VaR_99 = -2.33 * self.port_std * np.sqrt(self.trade_length)
         CVaR = self.LPM_1 / self.LPM_0
 
-        print(f'Portfolio daily average return is {round(self.port_mean, 2)}.')
-        print(f'Portfolio standard deviation is {round(self.port_std, 2)}.')
-        print(f'Daily average return for approximately 90% population is {round(self.inner_mean, 2)}.')
-        print(f'Downside daily probability is {round(self.LPM_0, 2)}.')
-        print(
-            f'There is 95% confidence that you will not lose more than {round(100 * VaR_95, 2)} % of your portfolio in a given {self.trade_length} period.')
-        print(
-            f'There is 99% confidence that you will not lose more than {round(100 * VaR_99, 2)} % of your portfolio in a given {self.trade_length} period.')
-        print(f'Expected loss that occur beyond the shortfall is {round(CVaR, 4)}.')
-
         # ----------------------------------------------------------------------- #
         # Drawdown
+        progress(20)
+
         port_performance_drawdown = self.final_portfolio.copy()
         port_performance_drawdown = port_performance_drawdown.clip(upper=0)
         port_performance_drawdown = port_performance_drawdown.drop(columns=['Sum', 'Accumulation'])
@@ -405,6 +524,8 @@ class backtest:
 
         # ----------------------------------------------------------------------- #
         # Monthly prod
+        progress(20)
+
         mon = []
         df_monthly = self.final_portfolio.copy()
         df_monthly = df_monthly.drop(index=df_monthly.index[0], axis=0)  # drop first row
@@ -427,38 +548,47 @@ class backtest:
 
         # ----------------------------------------------------------------------- #
         # Create figures in Express
+        progress(20)
 
         df_accum_fig1 = df_accum.astype(float)
-        df_accum_fig1.iloc[:,:-2] = (df_accum_fig1.iloc[:,:-2] * 100)
+        df_accum_fig1.iloc[:, :-2] = (df_accum_fig1.iloc[:, :-2] * 100)
         df_accum_fig1 = df_accum_fig1.round(2)
         fig1 = px.line(df_accum_fig1,
                        x=df_accum_fig1.index,
                        y=df_accum_fig1["Accumulation"],
                        hover_data=df_accum_fig1.columns[:-2])  # show all columns values excluding last 2
 
-        df_drawdown_fig2 = df_drawdown.astype(float)
-        df_drawdown_fig2.iloc[:,:-2] = (df_drawdown_fig2.iloc[:,:-2] * 100)
-        df_drawdown_fig2 = df_drawdown_fig2.round(2)
-        fig2 = px.line(df_drawdown_fig2,
-                       x=df_drawdown_fig2.index,
-                       y=df_drawdown_fig2["Accumulation"],
-                       hover_data=df_drawdown_fig2.columns[:-2])  # show all columns values excluding last 2
-
-        df_montly_fig3 = df_montly.astype(float)
-        df_montly_fig3 = df_montly_fig3.round(2)
-        fig3 = px.bar(df_montly_fig3,
-                      x=df_montly_fig3.index,
-                      y=df_montly_fig3["Result"])
-
         weights_df = self.portfolio_weights
         weights_df["Total Weights"] = weights_df.sum(axis=1)
         weights_df = weights_df[weights_df["Total Weights"] != 0]
         weights_df = weights_df.drop("Total Weights", axis=1)
         weights_df = abs(weights_df)
-        fig4 = px.area(weights_df * 100,
+        fig2 = px.area(weights_df * 100,
                        x=weights_df.index,
                        y=weights_df.columns)
 
+        df_montly_fig3 = df_montly.astype(float)
+        color = []
+        for i in df_montly_fig3["Result"]:
+            if i < 0:
+                color.append('red')
+            else:
+                color.append('green')
+        df_montly_fig3["color"] = color
+        df_montly_fig3 = df_montly_fig3.round(2)
+        fig3 = px.bar(df_montly_fig3,
+                      x=df_montly_fig3.index,
+                      y=df_montly_fig3["Result"],
+                      color=df_montly_fig3["color"],
+                      color_discrete_sequence=df_montly_fig3["color"].unique())
+
+        df_drawdown_fig2 = df_drawdown.astype(float)
+        df_drawdown_fig2.iloc[:, :-2] = (df_drawdown_fig2.iloc[:, :-2] * 100)
+        df_drawdown_fig2 = df_drawdown_fig2.round(2)
+        fig4 = px.line(df_drawdown_fig2,
+                       x=df_drawdown_fig2.index,
+                       y=df_drawdown_fig2["Sum"] * 100,
+                       hover_data=df_drawdown_fig2.columns[:-2])  # show all columns values excluding last 2
 
         # For as many traces that exist per Express figure, get the traces from each plot and store them in an array.
         # This is essentially breaking down the Express fig1, 2, 3 into it's traces
@@ -485,9 +615,10 @@ class backtest:
                                        shared_xaxes=False,
                                        shared_yaxes=False,
                                        subplot_titles=("Accumulative return",
-                                                       "Drawdown",
+                                                       "Weights distribution",
                                                        "Monthly return",
-                                                       "Weights distribution"))
+                                                       "Drawdown"
+                                                       ))
 
         for traces in figure1_traces:
             this_figure.append_trace(traces, row=1, col=1)
@@ -506,6 +637,25 @@ class backtest:
         this_figure.update_yaxes(tickprefix="%")
         this_figure['layout'].update(height=1400, title='Plotting results')
 
+        progress(20)
+        end_progress()
+
+        # ----------------------------------------------------------------------- #
+        # Printing stats
+
+        print(f'Portfolio daily average return is {round(self.port_mean, 2)}.')
+        print(f'Portfolio standard deviation is {round(self.port_std, 2)}.')
+        print(f'Daily average return for approximately 90% population is {round(self.inner_mean, 2)}.')
+        print(f'Downside daily probability is {round(self.LPM_0, 2)}.')
+        print(
+            f'There is 95% confidence that you will not lose more than {round(100 * VaR_95, 2)} % of your portfolio in a given {self.trade_length} period.')
+        print(
+            f'There is 99% confidence that you will not lose more than {round(100 * VaR_99, 2)} % of your portfolio in a given {self.trade_length} period.')
+        print(f'Expected loss that occur beyond the shortfall is {round(CVaR, 4)}.')
+
+        # ----------------------------------------------------------------------- #
+        # Printing plots
+
         this_figure.show()
 
     def puzzle_assembly(dic):
@@ -515,6 +665,9 @@ class backtest:
         :return:
             Combines several backrests results into one dataframe
         """
+        start_progress("Compiling your puzzle: ")
+
+        progress(50)
 
         names = dic.keys()
         empty_frame = pd.DataFrame()
@@ -528,6 +681,8 @@ class backtest:
         empty_frame = empty_frame.astype(float)
         empty_frame = empty_frame.round(4)
 
+        progress(50)
+        end_progress()
 
         return empty_frame
 
@@ -538,13 +693,17 @@ class backtest:
         :return:
             Combined backtests statistic and cumulative return of the portfolio
         """
+        start_progress("Working on puzzle results: ")
+
         empty_frame = data
         empty_frame = empty_frame.round(decimals=3)
         empty_frame = empty_frame.fillna(0)
-        q1 = empty_frame.iloc[:,:-2]
+        q1 = empty_frame.iloc[:, :-2]
         q2 = sum(abs(q1).sum(axis=1) == 0)
         # ----------------------------------------------------------------------- #
         # Stats
+        progress(50)
+
         pdr = empty_frame['Sum'] - 1
         port_mean = pdr.mean()
         port_std = pdr.std()
@@ -563,10 +722,11 @@ class backtest:
                   CVaR]
         frame = pd.DataFrame({'Indicators': list_1, 'Values': list_2})
         frame = frame.to_string(index=False)
-        print(frame)
 
         # ----------------------------------------------------------------------- #
         # Execution plot - Accumulation
+        progress(50)
+
         df_execution_fig1 = empty_frame.astype(float)
         df_execution_fig1.iloc[:, :-2] = (df_execution_fig1.iloc[:, :-2] * 100)
         df_execution_fig1 = df_execution_fig1.round(2)
@@ -577,10 +737,17 @@ class backtest:
                        hover_data=df_execution_fig1.columns[:-2])  # show all columns values excluding last 2
         fig1.update_layout(xaxis_title="Date")
         fig1.update_yaxes(tickprefix="%")
+
+        # ----------------------------------------------------------------------- #
+        # Printing stats
+        end_progress()
+        print(frame)
+
+        # ----------------------------------------------------------------------- #
+        # Printing plots
+
         fig1.show()
 
-
-
     def puzzle_plotting(data):
         """
         :param:
@@ -589,10 +756,14 @@ class backtest:
             Combines several backtests results into graphical representations
         """
 
+        start_progress("Working on puzzle plots: ")
+
         df = data
         df = df.round(decimals=3)
         df = df.fillna(0)
 
+        progress(20)
+
         # ----------------------------------------------------------------------- #
         # Stats
         empty_frame = df
@@ -606,7 +777,7 @@ class backtest:
         LPM_2 = pdr.clip(upper=0).std()
         if LPM_0 == 0:
             LPM_0 = 0.01
-        trade_length = len(pdr) -q2
+        trade_length = len(pdr) - q2
         VaR_95 = -1.65 * port_std * np.sqrt(trade_length)
         VaR_99 = -2.33 * port_std * np.sqrt(trade_length)
         CVaR = LPM_1 / LPM_0
@@ -616,24 +787,24 @@ class backtest:
                   CVaR]
         frame = pd.DataFrame({'Indicators': list_1, 'Values': list_2})
         frame = frame.to_string(index=False)
-        print(frame)
 
         # ----------------------------------------------------------------------- #
         # Drawdown
+        progress(20)
+
         port_performance_drawdown = df.copy()
         port_performance_drawdown = port_performance_drawdown.clip(upper=0)
         port_performance_drawdown = port_performance_drawdown.drop(columns=['Sum', 'Accumulation'])
-        port_performance_drawdown = abs(port_performance_drawdown)
+        #port_performance_drawdown = abs(port_performance_drawdown)
         port_performance_drawdown['Sum'] = port_performance_drawdown.sum(axis=1)
-        port_performance_drawdown['Sum'] = port_performance_drawdown['Sum'] + 1
-        port_performance_drawdown['Accumulation'] = (port_performance_drawdown['Sum'].cumprod() - 1) * 100
-        port_performance_drawdown['Accumulation'] = port_performance_drawdown['Accumulation'] * (-1)
-        port_performance_drawdown = port_performance_drawdown.round(decimals=3)
+        port_performance_drawdown['Accumulation'] = (port_performance_drawdown['Sum'].cumsum()) * 100
         df_drawdown = port_performance_drawdown
         df_drawdown = df_drawdown.round(decimals=3)
 
         # ----------------------------------------------------------------------- #
         # Monthly prod
+        progress(20)
+
         mon = []
         df_monthly = df.copy()
         df_monthly = df_monthly.drop(index=df.index[0], axis=0)  # drop first row
@@ -650,35 +821,45 @@ class backtest:
         com_frame = pd.DataFrame(index=d, data=v)
         com_frame = com_frame.sort_index()
         com_frame.index = com_frame.index.map(str)
-        #com_frame.index = pd.to_datetime(com_frame.index)
+        # com_frame.index = pd.to_datetime(com_frame.index)
         df_montly = com_frame
         df_montly.columns = ["Result"]
 
         # ----------------------------------------------------------------------- #
         # Create figures in Express
+        progress(20)
 
         df_accum_fig1 = df.astype(float)
-        df_accum_fig1.iloc[:,:-2] = (df_accum_fig1.iloc[:,:-2] * 100)
+        df_accum_fig1.iloc[:, :-2] = (df_accum_fig1.iloc[:, :-2] * 100)
         df_accum_fig1 = df_accum_fig1.round(2)
         fig1 = px.line(df_accum_fig1,
                        x=df_accum_fig1.index,
                        y=df_accum_fig1["Accumulation"],
                        hover_data=df_accum_fig1.columns[:-2])  # show all columns values excluding last 2
 
+        df_montly_fig3 = df_montly.astype(float)
+        color = []
+        for i in df_montly_fig3["Result"]:
+            if i < 0:
+                color.append('red')
+            else:
+                color.append('green')
+        df_montly_fig3["color"] = color
+        df_montly_fig3 = df_montly_fig3.round(2)
+        fig2 = px.bar(df_montly_fig3,
+                      x=df_montly_fig3.index,
+                      y=df_montly_fig3["Result"],
+                      color=df_montly_fig3["color"],
+                      color_discrete_sequence=df_montly_fig3["color"].unique())
 
         df_drawdown_fig2 = df_drawdown.astype(float)
-        df_drawdown_fig2.iloc[:,:-2] = (df_drawdown_fig2.iloc[:,:-2] * 100) * (-1)
+        df_drawdown_fig2.iloc[:, :-2] = (df_drawdown_fig2.iloc[:, :-2] * 100)
         df_drawdown_fig2 = df_drawdown_fig2.round(2)
-        fig2 = px.line(df_drawdown_fig2,
+        fig3 = px.line(df_drawdown_fig2,
                        x=df_drawdown_fig2.index,
-                       y=df_drawdown_fig2["Accumulation"],
+                       y=df_drawdown_fig2["Sum"] * 100,
                        hover_data=df_drawdown_fig2.columns[:-2])  # show all columns values excluding last 2
 
-        df_montly_fig3 = df_montly.astype(float)
-        df_montly_fig3 = df_montly_fig3.round(2)
-        fig3 = px.bar(df_montly_fig3,
-                      x=df_montly_fig3.index,
-                      y=df_montly_fig3["Result"])
 
         # For as many traces that exist per Express figure, get the traces from each plot and store them in an array.
         # This is essentially breaking down the Express fig1 into it's traces
@@ -700,8 +881,8 @@ class backtest:
                                        vertical_spacing=0.1,
                                        shared_xaxes=True,
                                        subplot_titles=("Puzzle accumulative return",
-                                                       "Puzzle Drawdown",
-                                                       "Puzzle monthly return"))
+                                                       "Puzzle monthly return",
+                                                       "Puzzle Drawdown"))
 
         for traces in figure1_traces:
             this_figure.append_trace(traces, row=1, col=1)
@@ -712,11 +893,24 @@ class backtest:
         for traces in figure3_traces:
             this_figure.append_trace(traces, row=3, col=1)
 
-        this_figure.update_layout(hovermode='x')
+        this_figure.update_layout(hovermode='x', showlegend=False)
         # Prefix y-axis tick labels with % sign
         this_figure.update_yaxes(tickprefix="%")
         this_figure['layout'].update(height=1400, title='Puzzle plotting results')
+
+        progress(20)
+        end_progress()
+
+        # ----------------------------------------------------------------------- #
+        # Printing stats
+
+        print(frame)
+
+        # ----------------------------------------------------------------------- #
+        # Printing plots
+
         this_figure.show()
 
+
 if __name__ == "__main__":
     backtest
